% This file tests all combinations of Ed1, Vd1, Ec1 y Ec2 between
% fixed ranges to verify feasibility/practical constraints described in the
% section "Determination of operational variables' values in the experimental
% design" of the paper. Each valid combination will correspond to a
% complete fractional factorial experimental design with 30 experiments
% (including center and extended experiments)

clear all;

% Iterations for testing the general feasibility/practical constraints with
% all combinations of operational variables
soluc = [];                     % Initializaton of solution vector
for i = 1:0.5:5                 % Range for Ed1(0): 1-5 %(v/v)
    Ed1_0 = i;
    for j=0.5:0.5:5             % Width from Ed1(0): 0.5-5 (increment=0.5)
        Ed1_2(1) = Ed1_0-j;     % Ed1_2(1) = Ed1(-2)
        Ed1_2(2) = Ed1_0+j;     % Ed1_2(2) = Ed1(+2)
        Ed1(1) = Ed1_0-j/2;     % Ed1(1) = Ed1(-1)
        Ed1(2) = Ed1_0+j/2;     % Ed1(2) = Ed1(+1)
        % Discard case if Ed1(-1/+1/-2+2) is out of practical limits
        if Ed1(1) < 1 || Ed1(2) > 7 || Ed1_2(1) < 1 || Ed1_2(2) > 7
            continue;
        end
        
        for i2 = 1:0.5:7                % Range for Vd1(0): 1-7 L
            Vd1_0 = i2;
            for j2 = 0.5:0.5:7          % Width from Vd1(0): 0.5-7 (increment=0.5)
                Vd1_2(1) = Vd1_0-j2;    % Vd1_2(1) = Vd1(-2)
                Vd1_2(2) = Vd1_0+j2;    % Vd1_2(2) = Vd1(+2)
                Vd1(1) = Vd1_0-j2/2;    % Vd1(1) = Vd1(-1)
                Vd1(2) = Vd1_0+j2/2;    % Vd1(2) = Vd1(+1)
                % Discard case if Vd1(-1/+1/-2+2) is out of practical limits
                if Vd1(1) < 1 || Vd1(2) > 7 || Vd1_2(1) < 1 || Vd1_2(2) > 7
                    continue;
                end
                % Maximum ethanol concentrations at loading stage for each
                % bioreactor
                Ec1_max_1=[];  % Vector to store Ec1_max for Vd1 and Ed1 (+1)/(-1) combinations
                Ec2_max_1=[];  % Vector to store Ec2_max for Vd1 and Ed1 (+1)/(-1) combinations
                Ec1_max_2=[];  % Vector to store Ec1_max for Vd1 y Ed1 (0)/(+2)/(-2) combinations
                Ec2_max_2=[];  % Vector to store Ec2_max for Vd1 y Ed1 (0)/(+2)/(-2) combinations
                % Calculation of all Ec1_max_1 and Ec2_max_1 values
                for i=1:2
                    for j=1:2
                        Ec1_max_1=[Ec1_max_1 ((8-Vd1(j))*Ed1(i)+Vd1(j)*12)/8];
                        Ec2_max_1=[Ec2_max_1 (Vd1(j)*Ed1(i)+(8-Vd1(j))*12)/8];
                    end
                end
                % Calculation of all Ec1_max_2 and Ec2_max_2 values
                % Ec1_max_0 and Ec2_max_0 store Ec1_max and Ec2_max with
                % Vd1(0) and Ed1(0), respectively
                Ec1_max_0 = ((8-Vd1_0)*Ed1_0+Vd1_0*12)/8;
                Ec2_max_0 = (Vd1_0*Ed1_0+(8-Vd1_0)*12)/8;
                Ec1_max_2 = [Ec1_max_2 Ec1_max_0];
                Ec2_max_2 = [Ec1_max_2 Ec2_max_0];
                % Combinations with Vd1(-2/+2) and Ed1(0)
                Ec1_max_2 = [Ec1_max_2 ((8-Vd1_2(2))*Ed1_0+Vd1_2(2)*12)/8];
                Ec2_max_2 = [Ec1_max_2 (Vd1_2(2)*Ed1_0+(8-Vd1_2(2))*12)/8];
                Ec1_max_2 = [Ec1_max_2 ((8-Vd1_2(1))*Ed1_0+Vd1_2(1)*12)/8];
                Ec2_max_2 = [Ec1_max_2 (Vd1_2(1)*Ed1_0+(8-Vd1_2(1))*12)/8];
                % Combinations with Vd1(0) and Ed1(-2/+2)
                Ec1_max_2 = [Ec1_max_2 ((8-Vd1_0)*Ed1_2(2)+Vd1_0*12)/8];
                Ec2_max_2 = [Ec1_max_2 (Vd1_0*Ed1_2(2)+(8-Vd1_0)*12)/8];
                Ec1_max_2 = [Ec1_max_2 ((8-Vd1_0)*Ed1_2(1)+Vd1_0*12)/8];
                Ec2_max_2 = [Ec1_max_2 (Vd1_0*Ed1_2(1)+(8-Vd1_0)*12)/8];
                % Calculation of Ec1_max_1, Ec2_max_1, Ec1_max_2 and
                % Ec2_max_2 minimum values
                min_Ec1_max_1 = min(Ec1_max_1);
                min_Ec2_max_1 = min(Ec2_max_1);
                min_Ec1_max_2 = min(Ec1_max_2);
                min_Ec2_max_2 = min(Ec2_max_2);

                for i3=1:0.5:5                  % Range for Ec1(0): 1-5 %(v/v)
                    Ec1_0 = i3;
                    for j3=0.5:0.5:5            % Width from Ec1(0): 0.5-5 (increment=0.5)
                        Ec1_2(1) = Ec1_0-j3;    % Ec1_2(1) = Ec1(-2)
                        Ec1_2(2) = Ec1_0+j3;    % Ec1_2(2) = Ec1(+2)
                        Ec1(1) = Ec1_0-j3/2;    % Ec1(1) = Ec1(-1)
                        Ec1(2) = Ec1_0+j3/2;    % Ec1(2) = Ec1(+1)
                        % Discard case if Ec1(-1/+1/-2+2) is out of practical limits
                        if Ec1(1) < 1 || Ec1(2) > 7 || Ec1_2(1) < 1 || Ec1_2(2) > 7
                           continue; 
                        end
                        % Constraints Ec1(-1/+1/-2/+2) must verify
                        cond1 = Ec1(1) < min_Ec1_max_1 && Ec1(2) < min_Ec1_max_1;
                        cond2 = Ec1(1) >= Ed1(2) && Ec1(2) >= Ed1(2);
                        cond3 = Ec1_0 < min_Ec1_max_2;
                        cond4 = Ec1_0 >= Ed1_2(2);
                        cond5 = Ec1_2(1) < Ec1_max_0 && Ec1_2(2) < Ec1_max_0;
                        cond6 = Ec1_2(1) >= Ed1_0 && Ec1_2(2) >= Ed1_0;
                        % If any of the previous constraints for Ec1 is not
                        % verified, then discard the case
                        OK_Ec1 = cond1 & cond2 & cond3 & cond4 & cond5 & cond6;
                        if ~OK_Ec1
                            continue;
                        end
                        
                        for i4=1:0.5:5                  % Range for Ec2(0): 1-5 %(v/v)
                            Ec2_0 = i4;
                            for j4 = 0.5:0.5:5          % Width from Ec2(0): 0.5-5 (increment=0.5)
                                Ec2_2(1) = Ec2_0-j4;    % Ec2_2(1) = Ec2(-2)
                                Ec2_2(2) = Ec2_0+j4;    % Ec2_2(1) = Ec2(+2)
                                Ec2(1) = Ec2_0-j4/2;    % Ec2(1) = Ec2(-1)
                                Ec2(2) = Ec2_0+j4/2;    % Ec2(2) = Ec2(+1)
                                % Discard case if Ec2(-1/+1/-2+2) is out of practical limits
                                if Ec2(1) < 1 || Ec2(2) > 7 || Ec2_2(1) < 1 || Ec2_2(2) > 7
                                    continue;
                                end
                                % Constraints Ec2(-1/+1/-2/+2) must verify
                                cond1 = Ec2(1) < min_Ec2_max_1 && Ec2(2) < min_Ec2_max_1;
                                cond2 = Ec2_0 < min_Ec2_max_2;
                                cond3 = Ec2_2(1) < Ec2_max_0 && Ec2_2(2) < Ec2_max_0;
                                % If any of the previous constraints for Ec2 is not
                                % verified, then discard the case
                                OK_Ec2 = cond1 & cond2 & cond3;
                                if ~OK_Ec2
                                    continue;
                                end
                                % The combination which satisfies all the
                                % constraints is stores in "soluc" vector
                                soluc = [soluc; Vd1_2(1) Vd1_2(2) Vd1(1) Vd1(2) Vd1_0 Ed1_2(1) Ed1_2(2) Ed1(1) Ed1(2) Ed1_0 Ec1_2(1) Ec1_2(2) Ec1(1) Ec1(2) Ec1_0 Ec2_2(1) Ec2_2(2) Ec2(1) Ec2(2) Ec2_0];
                            end
                        end
                    end 
                end
            end
        end
    end
end

% Save feasible combinations to a file (INITIAL PROGRAM EXECUTION)
save soluc.mat soluc;

% Reducing the number of feasible combinations imposing additional
% constraints (PASS 1):
% - Maximum variation for Vd1(-1/+1) and Ed1(-1/+1)
% - Vd1 >= 4 L
% - Ed1 <= 5 %(v/v)
soluc_tmp=[];
soluc2=[];          % Vector of reduced feasible combinations (fist pass)
% Only combinations with Vd1 >= 4 L and Ed1 <= 5 %(v/v) are selected for
% further processing
soluc_tmp = soluc(find(soluc(:,3) > 4 & soluc(:,7) <= 5),:);
fin = false;
while not(fin)
    max1 = max(soluc_tmp(:,4)-soluc_tmp(:,3));  % Maximum variation between Vd1(+1) y Vd1(-1)
    idx1 = find(soluc_tmp(:,4)-soluc_tmp(:,3) == max1);  % Index of combinations with maximum variation between Vd1(+1) y Vd1(-1)
    max2 = max(soluc_tmp(:,9)-soluc_tmp(:,8));  % Maximum variation between Ed1(+1) y Ed1(-1)
    idx2 = find(soluc_tmp(:,9)-soluc_tmp(:,8) == max2);  % Index of combinations with maximum variation between Ed1(+1) y ed1(-1)
    
    indices = intersect(idx1,idx2); % Indexes of common maximum variation of both variables

    % If there are no common indexes, then these combinations are "removed" from
    % "soluc_tmp" assigning zero to all variables' values
    if isempty(indices)
       soluc_tmp(idx1,:) = 0;
       soluc_tmp(idx2,:) = 0;
    % If there are common indexes, then the corresponding combinations are
    % the solution
    else
        fin = true;
        max_Vd1 = max1;
        max_Ed1 = max2;
    end
end
soluc2 = soluc_tmp(indices,:);
% Save feasible combinations to a file
save soluc2.mat soluc2;
% Save feasible combinations to an ASCII file (to import from Excel)
save soluc2.txt soluc2 -ascii;

% Reducing the number of feasible combinations imposing additional
% constraints (PASS 2):
% - Maximum variation for Ec1(-2/+2)
soluc_tmp = soluc2;
max1 = max(soluc_tmp(:,12)-soluc_tmp(:,11));  % Maximum variation between Ec1(+2) and Ec1(-2)
indices = find(soluc_tmp(:,12)-soluc_tmp(:,11) == max1);  % Index of combinations with maximum variation between Ec1(+2) and Ec1(-2)
soluc3 = soluc_tmp(indices,:);
% Save feasible combinations to a file
save soluc3.mat soluc3;
% Save feasible combinations to an ASCII file (to import from Excel)
save soluc3.txt soluc3 -ascii;


